/*
 * buttons.c
 *
 *  Created on: 24.03.2018
 *      Author: Rafa
 */

#include "stm32f0xx_hal.h"
#include "tsc.h"

#include "buttons.h"

#define CALIBRATE_ITERATIONS	5
#define THRESHOLD				500
#define	DEBONUCE_ITERATIONS		2

static struct button b1 = {.state=BUTTON_STATE_INIT, .port = TSC_GROUP2_IO4, .group=TSC_GROUP2_IDX};
static struct button b2 = {.state=BUTTON_STATE_INIT, .port = TSC_GROUP2_IO3, .group=TSC_GROUP2_IDX};
static struct button b3 = {.state=BUTTON_STATE_INIT, .port = TSC_GROUP1_IO1, .group=TSC_GROUP1_IDX};
static struct button b4 = {.state=BUTTON_STATE_INIT, .port = TSC_GROUP1_IO2, .group=TSC_GROUP1_IDX};
static struct button b5 = {.state=BUTTON_STATE_INIT, .port = TSC_GROUP2_IO1, .group=TSC_GROUP2_IDX};

struct button *buttons[BUTTONS] = {&b1, &b2, &b3, &b4, &b5};

static TSC_IOConfigTypeDef config = {
		.ChannelIOs = 0,
		.ShieldIOs = 0,
		.SamplingIOs = TSC_GROUP1_IO4|TSC_GROUP2_IO2
};

static void measure(struct button *b);
static void states(struct button *b);

void states_all(void)
{
	for (int i=0; i < BUTTONS; i++) {
		states(buttons[i]);
	}
}

void measure_all(void)
{
	for (int i=0; i < BUTTONS; i++) {
		measure(buttons[i]);
		//printf("%d\t", buttons[i]->value);
	}
	//printf("\r\n");
}

static void states(struct button *b)
{
	measure(b);
	switch(b->state) {
	case BUTTON_STATE_INIT:
		b->counter = CALIBRATE_ITERATIONS;
		b->no_touch_value = -1;
		b->state = BUTTON_STATE_CALIBRATE;
		b->clicked = 0;
		b->unclicked = 0;
		break;

	case BUTTON_STATE_CALIBRATE:
		if (b->value < b->no_touch_value) {
			b->no_touch_value = b->value;
		}
		b->counter--;
		if (b->counter == 0) {
			b->state = BUTTON_STATE_RELEASE;
		}
		break;
	case BUTTON_STATE_RELEASE:
		if (b->value < b->no_touch_value - THRESHOLD) {
			b->counter = DEBONUCE_ITERATIONS;
			b->state = BUTTON_STATE_DEB_DETECT;
		}
		break;
	case BUTTON_STATE_DEB_DETECT:
		if (b->value < b->no_touch_value - THRESHOLD) {
			b->counter--;
			if (b->counter == 0) {
				b->state = BUTTON_STATE_DETECT;
				b->clicked = 1;
			}
		} else {
			b->state = BUTTON_STATE_RELEASE;
		}
		break;
	case BUTTON_STATE_DETECT:
		if (b->value > b->no_touch_value - THRESHOLD) {
			b->counter = DEBONUCE_ITERATIONS;
			b->state = BUTTON_STATE_DEB_RELEASE;
		}
		break;
	case BUTTON_STATE_DEB_RELEASE:
		if (b->value > b->no_touch_value - THRESHOLD) {
			b->counter--;
			if (b->counter == 0) {
				b->state = BUTTON_STATE_RELEASE;
				b->unclicked = 1;
			}
		} else {
			b->state = BUTTON_STATE_DETECT;
		}
		break;
	default:
		b->state = BUTTON_STATE_RELEASE;
		break;
	}
}

static void measure(struct button *b)
{
	HAL_TSC_IODischarge(&htsc, ENABLE);

	config.ChannelIOs = b->port;
	if(HAL_TSC_IOConfig(&htsc, &config) != HAL_OK) {
		Error_Handler();
	}

	if(HAL_TSC_Start(&htsc) != HAL_OK) {
		Error_Handler();
	}
	while (HAL_TSC_GetState(&htsc) == HAL_TSC_STATE_BUSY);
	__HAL_TSC_CLEAR_FLAG(&htsc, (TSC_FLAG_EOA | TSC_FLAG_MCE));

	if (HAL_TSC_GroupGetStatus(&htsc, b->group) == TSC_GROUP_COMPLETED) {
		b->value = HAL_TSC_GroupGetValue(&htsc, b->group);
	}
}
